﻿using ProjectTank.DrawningObjects;
using ProjectTank.Generics;
using ProjectTank.MovementStrategy;
using System.Windows.Forms;
using Microsoft.VisualBasic.Logging;
using ProjectTank.Exceptions;
using Serilog;
using Log = Serilog.Log;

namespace ProjectTank
{
    public partial class FormTankCollection : Form
    {
        private readonly TanksGenericStorage _storage;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormTankCollection()
        {
            InitializeComponent();
            _storage = new TanksGenericStorage(pictureBoxCollection.Width,
                pictureBoxCollection.Height);
        }

        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();

            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i].Name);
            }

            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }

            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }

        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            Log.Information($"Добавлен набор: {textBoxStorageName.Text}");
        }

        /// <summary>
        /// Сортировка по типу
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSortByType_Click(object sender, EventArgs e) => CompareTanks(new TankCompareByType());
        /// <summary>
        /// Сортировка по цвету
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSortByColor_Click(object sender, EventArgs e) => CompareTanks(new TankCompareByColor());
        /// <summary>
        /// Сортировка по сравнителю
        /// </summary>
        /// <param name="comparer"></param>
        private void CompareTanks(IComparer<DrawningTankBase?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1) return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];

            if (obj == null) return;

            obj.Sort(comparer);
            pictureBoxCollection.Image = obj.ShowTanks();
        }


        private void ListBoxObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowTanks();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1) return;

            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo,
        MessageBoxIcon.Question) == DialogResult.Yes)
            {
                string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
                _storage.DelSet(name);
                ReloadObjects();
                Log.Information($"Удален набор: {name}");
            }
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddTank_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1) return;
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            FormTankConfig form = new();
            form.Show();
            Action<DrawningTankBase>? tankDelegate = new((t) =>
            {
                try
                {
                    bool q = (obj + t);
                    MessageBox.Show("Объект добавлен");
                    Log.Information($"Добавлен объект в коллекцию {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    pictureBoxCollection.Image = obj.ShowTanks();
                    t.ChangePictureBoxSize(pictureBoxCollection.Width, pictureBoxCollection.Height);
                }
                catch (StorageOverflowException ex)
                {
                    Log.Warning($"Коллекция {listBoxStorages.SelectedItem.ToString() ?? string.Empty} переполнена");
                    MessageBox.Show(ex.Message);
                }
                catch (ArgumentException ex)
                {
                    Log.Warning($"Добавляемый объект уже существует в коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    MessageBox.Show("Добавляемый объект уже существует в коллекции");
                }
            });

            form.AddEvent(tankDelegate);
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveTank_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            if (MessageBox.Show("Удалить объект?", "Удаление",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            try
            {
                int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
                var q = obj - pos;
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowTanks();
                Log.Information($"Удален объект из коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty} по номеру {pos}");
            }
            catch (TankNotFoundException exception)
            {
                Log.Warning($"Не получилось удалить объект из коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                MessageBox.Show(exception.Message);
            }
            catch (FormatException ex)
            {
                Log.Warning("Было введено не-число при удалении объекта из набора");
                MessageBox.Show("Введите число");
            }
        }

        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1) return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            pictureBoxCollection.Image = obj.ShowTanks();
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно",
                        "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    Log.Information($"Файл {saveFileDialog.FileName} успешно сохранен");
                }
                catch (Exception exception)
                {
                    Log.Warning("Не удалось сохранить файл");
                    MessageBox.Show($"Не сохранилось: {exception.Message}", "Результат",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно",
                        "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    Log.Information($"Файл {openFileDialog.FileName} успешно загружен");
                    foreach (var collection in _storage.Keys)
                    {
                        listBoxStorages.Items.Add(collection);
                    }
                }
                catch (Exception exception)
                {
                    Log.Warning("Не удалось загрузить");
                    MessageBox.Show($"Не загрузилось: {exception.Message}", "Результат",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
