using ProjectTank.DrawningObjects;
using ProjectTank.Drawnings;
using ProjectTank.MovementStrategy;
using System.Text;

namespace ProjectTank.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class TanksGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<TanksCollectionInfo,
            TanksGenericCollection<DrawningTankBase, DrawningObjectTank>> _tankStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<TanksCollectionInfo> Keys => _tankStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public TanksGenericStorage(int pictureWidth, int pictureHeight)
        {
            _tankStorages = new Dictionary<TanksCollectionInfo,
                TanksGenericCollection<DrawningTankBase, DrawningObjectTank>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            _tankStorages.Add(new TanksCollectionInfo(name, string.Empty),
                new TanksGenericCollection<DrawningTankBase, DrawningObjectTank>(_pictureWidth, _pictureHeight));
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (!_tankStorages.ContainsKey(new TanksCollectionInfo(name, string.Empty)))
                return;
            _tankStorages.Remove(new TanksCollectionInfo(name, string.Empty));
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public TanksGenericCollection<DrawningTankBase, DrawningObjectTank>? this[string ind]
        {
            get
            {
                TanksCollectionInfo indObj = new TanksCollectionInfo(ind, string.Empty);
                if (_tankStorages.ContainsKey(indObj))
                    return _tankStorages[indObj];
                return null;
            }
        }

        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<TanksCollectionInfo, TanksGenericCollection<DrawningTankBase, DrawningObjectTank>> record in _tankStorages)
            {
                StringBuilder records = new();
                foreach (DrawningTankBase? elem in record.Value.GetTanks)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key.Name}{_separatorForKeyValue}{records}");
            }
            
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }

            string toWrite = $"TanksStorage{Environment.NewLine}{data}";
            var strs = toWrite.Split(new char[] { '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);

            using (StreamWriter sw = new(filename))
            {
                foreach (var str in strs)
                {
                    sw.WriteLine(str);
                }
            }

            return true;
        }
        /// <summary>
        /// Загрузка информации по танкам в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new IOException("Файл не найден");
            }

            using (StreamReader sr = new(filename))
            {
                string str = sr.ReadLine();
                var strs = str.Split(new char[] { '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);
                if (strs == null || strs.Length == 0)
                {
                    throw new IOException("Нет данных для загрузки");
                }

                if (!strs[0].StartsWith("TanksStorage"))
                {
                    //если нет такой записи, то это не те данные
                    throw new IOException("Неверный формат данных");
                }
                
                _tankStorages.Clear();
                
                do
                {
                    string[] record = str.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        str = sr.ReadLine();
                        continue;
                    }
                    TanksGenericCollection<DrawningTankBase, DrawningObjectTank> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords,
                    StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        DrawningTankBase? tank =
                        elem?.CreateDrawningTankBase(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (tank != null)
                        {
                            if (!(collection + tank))
                            {
                                return false;
                            }
                        }
                    }
                    _tankStorages.Add(new TanksCollectionInfo(record[0], string.Empty), collection);

                    str = sr.ReadLine();
                } while (str != null);
            }
            return true;
        }
    }
}